/************************************************************************************************
PROJECT : simple_draw
FILE    : $Id: main.c 15225 2017-11-21 12:01:29Z matthias.nippert $
============================================================================ 
DESCRIPTION
============================================================================
                            C O P Y R I G H T
============================================================================
                       Copyright (c) 2013 - 2014
                                  by 
                       Renesas Electronics (Europe) GmbH.
                           Arcadiastrasse 10
                          D-40472 Duesseldorf
                               Germany
                          All rights reserved.
=============================================================================
Purpose: only for testing, not for mass production

DISCLAIMER

LICENSEE has read, understood and accepted the terms and conditions defined in
the license agreement, especially the usage rights. In any case, it is
LICENSEE's responsibility to make sure that any user of the software complies
with the terms and conditions of the signed license agreement.

SAMPLE CODE is not part of the licensed software, as such it must not be used in
mass-production applications. It can only be used for evaluation and
demonstration purposes at customer's premises listed in the signed license
agreement.
****************************************************************************
*/

/*******************************************************************************
  Section: Includes
*/

#include "r_typedefs.h"        /* Renesas basic types, e.g. uint32_t */
//#include "r_bsp_api.h"         /* Board support package */
//#include "r_bsp_hmi_api.h"     /* Board support package - HMI (Buttons + Rotary) */
#include "r_ddb_api.h"
#include "r_cdi_api.h"
#include "r_wm_api.h"
#include "davehd_driver.h"

#include "main.h"
//#include "wm.h"
//#include "mem_init.h"
//#include "r_util_dhd.h"

//#include "config.h"
#include "r_drw2d_os.h"
#include "r_drw2d_api.h"
#include "r_config_drw2d.h"
#include "r_drw2d_ctx_dhd.h"

#include "r_jcua_api.h"
//#include "r_bsp_stdio_api.h"

#include "r_gpio_api.h"
#include "r_dev_pin_types.h"
#include "r_dev_api.h"
#include "sflash.h"
#include "sfma_ISSI.h"

#include "Graphic.h"

#include "Sys_Scheduler.h"
/***********************************************************
  Section: Local Defines
*/

/* bitmap widths */
#define LOC_KING_W    (30)
#define LOC_GEAR_W    (104)
/* bitmap heights */
#define LOC_KING_H    (30)
#define LOC_GEAR_H    (104)
/* bitmap sizes */
#define LOC_KING_SIZE (LOC_KING_W * LOC_KING_H * 4)
#define LOC_GEAR_SIZE (LOC_GEAR_W * LOC_GEAR_H * 4)

#define LOC_SF_BMP_W   (64)
#define LOC_SF_BMP_H   (64)


#if (defined(BSP_INIT_OCTA_FLASH_R) || defined(BSP_INIT_OCTA_RAM_RW) || defined(BSP_INIT_OCTA_PIN))
/* OCTA RAM/Flash */
#define LOC_EX_MEM_TYPE   (1)
#elif (defined(BSP_INIT_HYPB_FLASH_R) || defined(BSP_INIT_HYPB_RAM_RW) || defined(BSP_INIT_HYPB_PIN))
/* Hyper RAM/Flash */
#define LOC_EX_MEM_TYPE   (2)
#else
/* Serial Flash */
#define LOC_EX_MEM_TYPE   (0)
#endif

/***********************************************************
  Section: Local Variables
*/

/***********************************************************

  Variable: loc_WmWindow

  Pointer to the Window handles
*/


static const uint8_t ABS[] = {
0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x5,0x0,0xef,0xff,0x2a,0x0,0xef,0xff,0x31,0x0,0xef,0xff,0x9,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x2,0x0,0xef,0xff,0x16,0x0,0xf0,0xff,0x2e,0x0,0xf0,0xff,0x2c,0x0,0xef,0xff,0x25,0x0,0xef,0xff,0x25,0x0,0xf0,0xff,0x26,0x0,0xef,0xff,0x26,0x0,0xef,0xff,0x26,0x0,0xef,0xff,0x25,0x0,0xef,0xff,0x29,0x0,0xf0,0xff,0x2e,0x0,0xf0,0xff,0x21,0x0,0xef,0xff,0x6,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x13,0x0,0xf0,0xff,0x2a,0x0,0xf0,0xff,0x2d,0x0,0xef,0xff,0x25,0x0,0xef,0xff,0x25,0x0,0xf0,0xff,0x26,0x0,0xf0,0xff,0x25,0x0,0xef,0xff,0x25,0x0,0xef,0xff,0x2b,0x0,0xf0,0xff,0x2d,0x0,0xf0,0xff,0x14,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x1e,0x0,0xf0,0xff,0x84,0x0,0xf0,0xff,0x99,0x0,0xef,0xff,0x34,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0xc,0x0,0xef,0xff,0x51,0x0,0xf0,0xff,0x92,0x0,0xf0,0xff,0x97,0x0,0xef,0xff,0x91,0x0,0xef,0xff,0x91,0x0,0xf0,0xff,0x90,0x0,0xf0,0xff,0x90,0x0,0xef,0xff,0x90,0x0,0xef,0xff,0x8f,0x0,0xef,0xff,0x95,0x0,0xef,0xff,0x9f,0x0,0xef,0xff,0x8c,0x0,0xef,0xff,0x4a,0x0,0xf0,0xff,0x7,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x2a,0x0,0xef,0xff,0x70,0x0,0xf0,0xff,0x9b,0x0,0xf0,0xff,0x9c,0x0,0xef,0xff,0x90,0x0,0xf0,0xff,0x8f,0x0,0xf0,0xff,0x90,0x0,0xf0,0xff,0x8f,0x0,0xef,0xff,0x8f,0x0,0xef,0xff,0x99,0x0,0xf0,0xff,0x9e,0x0,0xf0,0xff,0x75,0x0,0xef,0xff,0x32,0x0,0xef,0xff,0x3,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x3,0x0,0xf0,0xff,0x6a,0x0,0xf0,0xff,0xed,0x0,0xf0,0xff,0xfd,0x0,0xf0,0xff,0x94,0x0,0xef,0xff,0x10,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x1e,0x0,0xef,0xff,0x9a,0x0,0xf0,0xff,0xf3,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xd3,0x0,0xf0,0xff,0x6e,0x0,0xf0,0xff,0x16,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x1,0x0,0xf0,0xff,0x40,0x0,0xef,0xff,0xad,0x0,0xef,0xff,0xf3,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xf8,0x0,0xef,0xff,0xbe,0x0,0xef,0xff,0x57,0x0,0xf0,0xff,0x3,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x30,0x0,0xef,0xff,0xc4,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0xe4,0x0,0xef,0xff,0x52,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x26,0x0,0xef,0xff,0xb4,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xe2,0x0,0xef,0xff,0xbc,0x0,0xef,0xff,0xbc,0x0,0xf0,0xff,0xc5,0x0,0xf0,0xff,0xc4,0x0,0xf0,0xff,0xc4,0x0,0xef,0xff,0xc4,0x0,0xef,0xff,0xc1,0x0,0xef,0xff,0xba,0x0,0xef,0xff,0xc7,0x0,0xef,0xff,0xf8,0x0,0xf0,0xff,0xf0,0x0,0xf0,0xff,0x76,0x0,0xf0,0xff,0xf,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x3,0x0,0xef,0xff,0x3a,0x0,0xf0,0xff,0xc0,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xda,0x0,0xf0,0xff,0xba,0x0,0xf0,0xff,0xbd,0x0,0xef,0xff,0xc4,0x0,0xf0,0xff,0xc4,0x0,0xf0,0xff,0xc4,0x0,0xf0,0xff,0xc4,0x0,0xef,0xff,0xc4,0x0,0xef,0xff,0xc0,0x0,0xf0,0xff,0xbb,0x0,0xf0,0xff,0xd1,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xd9,0x0,0xf0,0xff,0x5e,0x0,0xf0,0xff,0x14,0x0,0xf0,0xff,0x4,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x6,0x0,0xf0,0xff,0x84,0x0,0xef,0xff,0xf7,0x0,0xf0,0xff,0xb9,0x0,0xf0,0xff,0xa4,0x0,0xef,0xff,0xf6,0x0,0xef,0xff,0xae,0x0,0xf0,0xff,0x17,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x26,0x0,0xf0,0xff,0xba,0x0,0xef,0xff,0xf6,0x0,0xef,0xff,0x87,0x0,0xef,0xff,0x23,0x0,0xef,0xff,0x20,0x0,0xf0,0xff,0x2e,0x0,0xf0,0xff,0x2c,0x0,0xef,0xff,0x2c,0x0,0xef,0xff,0x2c,0x0,0xf0,0xff,0x29,0x0,0xf0,0xff,0x20,0x0,0xf0,0xff,0x30,0x0,0xef,0xff,0x82,0x0,0xef,0xff,0xec,0x0,0xef,0xff,0xdd,0x0,0xf0,0xff,0x4f,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x1d,0x0,0xf0,0xff,0xa6,0x0,0xf0,0xff,0xfd,0x0,0xf0,0xff,0xb8,0x0,0xf0,0xff,0x4a,0x0,0xf0,0xff,0x20,0x0,0xef,0xff,0x24,0x0,0xf0,0xff,0x2d,0x0,0xf0,0xff,0x2d,0x0,0xef,0xff,0x2c,0x0,0xef,0xff,0x2d,0x0,0xf0,0xff,0x2d,0x0,0xef,0xff,0x27,0x0,0xf0,0xff,0x20,0x0,0xf0,0xff,0x3e,0x0,0xef,0xff,0xad,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xeb,0x0,0xf0,0xff,0x88,0x0,0xf0,0xff,0x32,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x31,0x0,0xf0,0xff,0xce,0x0,0xf0,0xff,0xed,0x0,0xf0,0xff,0x4b,0x0,0xf0,0xff,0x2d,0x0,0xf0,0xff,0xd3,0x0,0xef,0xff,0xec,0x0,0xf0,0xff,0x53,0x0,0xf0,0xff,0x1,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x27,0x0,0xf0,0xff,0xbf,0x0,0xef,0xff,0xf0,0x0,0xef,0xff,0x63,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x9,0x0,0xef,0xff,0x9a,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0x8e,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x37,0x0,0xf0,0xff,0xe7,0x0,0xf0,0xff,0xec,0x0,0xef,0xff,0x42,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x39,0x0,0xef,0xff,0xb7,0x0,0xf0,0xff,0xf5,0x0,0xf0,0xff,0xc7,0x0,0xf0,0xff,0x54,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x5,0x0,0xef,0xff,0x7d,0x0,0xf0,0xff,0xfa,0x0,0xf0,0xff,0xb5,0x0,0xf0,0xff,0x1a,0x0,0xf0,0xff,0x9,0x0,0xf0,0xff,0x93,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xa9,0x0,0xef,0xff,0x16,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x27,0x0,0xf0,0xff,0xbf,0x0,0xf0,0xff,0xf1,0x0,0xf0,0xff,0x63,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x1,0x0,0xf0,0xff,0x8f,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0x98,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x39,0x0,0xf0,0xff,0xec,0x0,0xef,0xff,0xe8,0x0,0xef,0xff,0x35,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x2,0x0,0xef,0xff,0x3c,0x0,0xef,0xff,0x71,0x0,0xef,0xff,0x5c,0x0,0xef,0xff,0x28,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x26,0x0,0xef,0xff,0xc4,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0x77,0x0,0xf0,0xff,0x4,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x45,0x0,0xf0,0xff,0xe9,0x0,0xf0,0xff,0xeb,0x0,0xef,0xff,0x4e,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x27,0x0,0xef,0xff,0xbe,0x0,0xef,0xff,0xf2,0x0,0xf0,0xff,0x6b,0x0,0xf0,0xff,0x9,0x0,0xf0,0xff,0x6,0x0,0xf0,0xff,0x8,0x0,0xef,0xff,0x8,0x0,0xf0,0xff,0x8,0x0,0xf0,0xff,0x8,0x0,0xef,0xff,0x8,0x0,0xef,0xff,0x6,0x0,0xf0,0xff,0xa,0x0,0xef,0xff,0x4f,0x0,0xf0,0xff,0xd3,0x0,0xf0,0xff,0xec,0x0,0xef,0xff,0x61,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x29,0x0,0xf0,0xff,0xc5,0x0,0xef,0xff,0xfe,0x0,0xef,0xff,0x90,0x0,0xef,0xff,0x1b,0x0,0xef,0xff,0x5,0x0,0xf0,0xff,0x7,0x0,0xf0,0xff,0x8,0x0,0xf0,0xff,0x8,0x0,0xef,0xff,0x8,0x0,0xef,0xff,0x8,0x0,0xf0,0xff,0x8,0x0,0xef,0xff,0x8,0x0,0xef,0xff,0x9,0x0,0xef,0xff,0x7,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x8,0x0,0xef,0xff,0x6,0x0,0xef,0xff,0x2,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x6,0x0,0xef,0xff,0x75,0x0,0xef,0xff,0xf4,0x0,0xf0,0xff,0xc6,0x0,0xf0,0xff,0x34,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x16,0x0,0xf0,0xff,0xa0,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xa3,0x0,0xf0,0xff,0x17,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x20,0x0,0xef,0xff,0xaa,0x0,0xef,0xff,0xf8,0x0,0xf0,0xff,0xbd,0x0,0xf0,0xff,0x80,0x0,0xf0,0xff,0x80,0x0,0xef,0xff,0x8a,0x0,0xef,0xff,0x88,0x0,0xf0,0xff,0x88,0x0,0xf0,0xff,0x88,0x0,0xef,0xff,0x86,0x0,0xef,0xff,0x80,0x0,0xf0,0xff,0x8e,0x0,0xef,0xff,0xd4,0x0,0xef,0xff,0xee,0x0,0xef,0xff,0x81,0x0,0xef,0xff,0x12,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x9,0x0,0xf0,0xff,0x67,0x0,0xf0,0xff,0xeb,0x0,0xf0,0xff,0xf5,0x0,0xef,0xff,0xac,0x0,0xef,0xff,0x7f,0x0,0xf0,0xff,0x7f,0x0,0xf0,0xff,0x88,0x0,0xf0,0xff,0x88,0x0,0xef,0xff,0x88,0x0,0xef,0xff,0x87,0x0,0xf0,0xff,0x87,0x0,0xef,0xff,0x8c,0x0,0xef,0xff,0x90,0x0,0xef,0xff,0x80,0x0,0xf0,0xff,0x4f,0x0,0xf0,0xff,0x11,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x27,0x0,0xf0,0xff,0xc6,0x0,0xf0,0xff,0xf6,0x0,0xf0,0xff,0x6b,0x0,0xf0,0xff,0x5,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x3e,0x0,0xef,0xff,0xdc,0x0,0xef,0xff,0xec,0x0,0xf0,0xff,0x4f,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x1b,0x0,0xf0,0xff,0x98,0x0,0xf0,0xff,0xfa,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xf7,0x0,0xf0,0xff,0xf9,0x0,0xef,0xff,0xfd,0x0,0xef,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xef,0xff,0xfb,0x0,0xef,0xff,0xf8,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xdb,0x0,0xef,0xff,0x3e,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x12,0x0,0xf0,0xff,0x70,0x0,0xf0,0xff,0xd6,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xfa,0x0,0xf0,0xff,0xf7,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xef,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xef,0xff,0xfe,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xf8,0x0,0xef,0xff,0xdc,0x0,0xf0,0xff,0x81,0x0,0xf0,0xff,0x13,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x7,0x0,0xf0,0xff,0x72,0x0,0xf0,0xff,0xef,0x0,0xf0,0xff,0xa5,0x0,0xf0,0xff,0x16,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x1,0x0,0xf0,0xff,0x81,0x0,0xf0,0xff,0xf7,0x0,0xf0,0xff,0x9d,0x0,0xef,0xff,0x15,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x20,0x0,0xf0,0xff,0xa7,0x0,0xef,0xff,0xf8,0x0,0xf0,0xff,0xc9,0x0,0xf0,0xff,0x94,0x0,0xf0,0xff,0x93,0x0,0xef,0xff,0x9d,0x0,0xf0,0xff,0x9c,0x0,0xf0,0xff,0x9c,0x0,0xf0,0xff,0x9c,0x0,0xef,0xff,0x9a,0x0,0xef,0xff,0x94,0x0,0xf0,0xff,0x9e,0x0,0xef,0xff,0xda,0x0,0xef,0xff,0xef,0x0,0xf0,0xff,0x7f,0x0,0xf0,0xff,0x12,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0xa,0x0,0xf0,0xff,0x43,0x0,0xef,0xff,0x89,0x0,0xef,0xff,0xa4,0x0,0xf0,0xff,0xa1,0x0,0xf0,0xff,0x9c,0x0,0xf0,0xff,0x9c,0x0,0xef,0xff,0x9c,0x0,0xf0,0xff,0x9d,0x0,0xf0,0xff,0x9d,0x0,0xf0,0xff,0x97,0x0,0xf0,0xff,0x91,0x0,0xf0,0xff,0xab,0x0,0xef,0xff,0xf4,0x0,0xf0,0xff,0xf2,0x0,0xf0,0xff,0x83,0x0,0xf0,0xff,0x1c,0x0,0xf0,0xff,0x2,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x23,0x0,0xf0,0xff,0xbe,0x0,0xf0,0xff,0xfd,0x0,0xf0,0xff,0x9d,0x0,0xf0,0xff,0x47,0x0,0xef,0xff,0x49,0x0,0xf0,0xff,0x54,0x0,0xf0,0xff,0x54,0x0,0xf0,0xff,0x4d,0x0,0xf0,0xff,0x46,0x0,0xf0,0xff,0x82,0x0,0xf0,0xff,0xf2,0x0,0xef,0xff,0xe2,0x0,0xef,0xff,0x48,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x27,0x0,0xef,0xff,0xbd,0x0,0xef,0xff,0xf3,0x0,0xf0,0xff,0x74,0x0,0xf0,0xff,0xc,0x0,0xef,0xff,0xa,0x0,0xef,0xff,0x14,0x0,0xf0,0xff,0x12,0x0,0xf0,0xff,0x12,0x0,0xef,0xff,0x12,0x0,0xf0,0xff,0x10,0x0,0xef,0xff,0xa,0x0,0xf0,0xff,0x13,0x0,0xef,0xff,0x55,0x0,0xef,0xff,0xd5,0x0,0xf0,0xff,0xe7,0x0,0xf0,0xff,0x60,0x0,0xf0,0xff,0x1,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x1,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x9,0x0,0xef,0xff,0x18,0x0,0xf0,0xff,0x17,0x0,0xf0,0xff,0x12,0x0,0xf0,0xff,0x12,0x0,0xef,0xff,0x12,0x0,0xf0,0xff,0x13,0x0,0xef,0xff,0x13,0x0,0xf0,0xff,0xe,0x0,0xf0,0xff,0x9,0x0,0xf0,0xff,0x1e,0x0,0xef,0xff,0x81,0x0,0xef,0xff,0xed,0x0,0xf0,0xff,0xf0,0x0,0xf0,0xff,0x91,0x0,0xf0,0xff,0x33,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x4,0x0,0xf0,0xff,0x5b,0x0,0xef,0xff,0xe9,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xf1,0x0,0xf0,0xff,0xdd,0x0,0xef,0xff,0xe6,0x0,0xf0,0xff,0xf0,0x0,0xf0,0xff,0xf0,0x0,0xf0,0xff,0xea,0x0,0xef,0xff,0xdf,0x0,0xf0,0xff,0xe8,0x0,0xf0,0xff,0xff,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0x88,0x0,0xef,0xff,0xd,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x27,0x0,0xef,0xff,0xc1,0x0,0xf0,0xff,0xf0,0x0,0xf0,0xff,0x63,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x8c,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0x9f,0x0,0xf0,0xff,0x4,0x0,0xf0,0xff,0x10,0x0,0xf0,0xff,0x50,0x0,0xf0,0xff,0x48,0x0,0xf0,0xff,0xa,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x21,0x0,0xef,0xff,0xb0,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xeb,0x0,0xf0,0xff,0x71,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x21,0x0,0xf0,0xff,0xb2,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0xd5,0x0,0xf0,0xff,0xb8,0x0,0xf0,0xff,0xca,0x0,0xf0,0xff,0xd4,0x0,0xf0,0xff,0xd2,0x0,0xf0,0xff,0xd2,0x0,0xf0,0xff,0xd3,0x0,0xef,0xff,0xcf,0x0,0xf0,0xff,0xbc,0x0,0xef,0xff,0xc5,0x0,0xef,0xff,0xf7,0x0,0xef,0xff,0xd1,0x0,0xef,0xff,0x40,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x26,0x0,0xf0,0xff,0xbe,0x0,0xf0,0xff,0xf1,0x0,0xf0,0xff,0x63,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0xf,0x0,0xf0,0xff,0x9f,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0x91,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x2c,0x0,0xef,0xff,0xc5,0x0,0xef,0xff,0xd5,0x0,0xf0,0xff,0x4f,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x34,0x0,0xf0,0xff,0xc4,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xd4,0x0,0xf0,0xff,0x62,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x2,0x0,0xef,0xff,0x61,0x0,0xef,0xff,0xf3,0x0,0xf0,0xff,0xdc,0x0,0xf0,0xff,0x54,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x2b,0x0,0xf0,0xff,0x34,0x0,0xef,0xff,0x32,0x0,0xef,0xff,0x32,0x0,0xef,0xff,0x33,0x0,0xf0,0xff,0x30,0x0,0xef,0xff,0x1f,0x0,0xef,0xff,0x34,0x0,0xef,0xff,0xb5,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0x8e,0x0,0xf0,0xff,0xd,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x26,0x0,0xef,0xff,0xbb,0x0,0xef,0xff,0xf4,0x0,0xf0,0xff,0x7e,0x0,0xf0,0xff,0x1d,0x0,0xef,0xff,0x19,0x0,0xef,0xff,0x1d,0x0,0xef,0xff,0x1d,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x1b,0x0,0xf0,0xff,0x17,0x0,0xf0,0xff,0x21,0x0,0xf0,0xff,0x79,0x0,0xf0,0xff,0xef,0x0,0xef,0xff,0xe2,0x0,0xef,0xff,0x4d,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x25,0x0,0xef,0xff,0xb9,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xc3,0x0,0xef,0xff,0x40,0x0,0xef,0xff,0x18,0x0,0xf0,0xff,0x19,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x1d,0x0,0xef,0xff,0x1d,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x1d,0x0,0xf0,0xff,0x1a,0x0,0xf0,0xff,0x17,0x0,0xf0,0xff,0x33,0x0,0xf0,0xff,0xa3,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0xd7,0x0,0xf0,0xff,0x69,0x0,0xf0,0xff,0x21,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x5,0x0,0xf0,0xff,0x3a,0x0,0xef,0xff,0xcb,0x0,0xef,0xff,0xfc,0x0,0xf0,0xff,0x86,0x0,0xf0,0xff,0xc,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x5b,0x0,0xf0,0xff,0xe9,0x0,0xef,0xff,0xe6,0x0,0xf0,0xff,0x58,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x24,0x0,0xef,0xff,0xb7,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xd8,0x0,0xf0,0xff,0xa5,0x0,0xef,0xff,0xa6,0x0,0xef,0xff,0xb1,0x0,0xef,0xff,0xaf,0x0,0xf0,0xff,0xaf,0x0,0xf0,0xff,0xb0,0x0,0xf0,0xff,0xab,0x0,0xf0,0xff,0xa2,0x0,0xf0,0xff,0xb7,0x0,0xef,0xff,0xf1,0x0,0xef,0xff,0xf5,0x0,0xf0,0xff,0x80,0x0,0xf0,0xff,0xb,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x4,0x0,0xf0,0xff,0x4a,0x0,0xf0,0xff,0xce,0x0,0xef,0xff,0xff,0x0,0xf0,0xff,0xd6,0x0,0xef,0xff,0xa7,0x0,0xef,0xff,0xa4,0x0,0xf0,0xff,0xaf,0x0,0xf0,0xff,0xb1,0x0,0xef,0xff,0xaf,0x0,0xf0,0xff,0xb1,0x0,0xf0,0xff,0xaf,0x0,0xf0,0xff,0xa7,0x0,0xf0,0xff,0xa3,0x0,0xf0,0xff,0xd5,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xd6,0x0,0xf0,0xff,0x5c,0x0,0xef,0xff,0x9,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xf0,0xff,0x20,0x0,0xf0,0xff,0x80,0x0,0xef,0xff,0xff,0x0,0xef,0xff,0xd8,0x0,0xef,0xff,0x3a,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x18,0x0,0xf0,0xff,0xac,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0x9d,0x0,0xf0,0xff,0x7,0x0,0xf0,0xff,0x1a,0x0,0xf0,0xff,0xa1,0x0,0xef,0xff,0xf8,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0xf9,0x0,0xef,0xff,0xfa,0x0,0xef,0xff,0xfd,0x0,0xef,0xff,0xfc,0x0,0xef,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfb,0x0,0xf0,0xff,0xfe,0x0,0xef,0xff,0xdc,0x0,0xef,0xff,0x7a,0x0,0xf0,0xff,0x19,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x3,0x0,0xf0,0xff,0x44,0x0,0xf0,0xff,0xb1,0x0,0xf0,0xff,0xf9,0x0,0xf0,0xff,0xfe,0x0,0xf0,0xff,0xfa,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfd,0x0,0xef,0xff,0xfc,0x0,0xf0,0xff,0xfd,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xfb,0x0,0xf0,0xff,0xfc,0x0,0xf0,0xff,0xff,0x0,0xf0,0xff,0xca,0x0,0xf0,0xff,0x50,0x0,0xf0,0xff,0x5,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0xef,0xff,0x24,0x0,0xf0,0xff,0x73,0x0,0xf0,0xff,0xbc,0x0,0xf0,0xff,0x7c,0x0,0xef,0xff,0xf,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x1,0x0,0xf0,0xff,0x57,0x0,0xf0,0xff,0xb9,0x0,0xf0,0xff,0x85,0x0,0xf0,0xff,0xd,0x0,0xf0,0xff,0xb,0x0,0xf0,0xff,0x62,0x0,0xf0,0xff,0xac,0x0,0xef,0xff,0xb4,0x0,0xef,0xff,0xb3,0x0,0xf0,0xff,0xb3,0x0,0xf0,0xff,0xb1,0x0,0xef,0xff,0xb1,0x0,0xef,0xff,0xb1,0x0,0xf0,0xff,0xb1,0x0,0xf0,0xff,0xb5,0x0,0xf0,0xff,0xbe,0x0,0xf0,0xff,0xad,0x0,0xef,0xff,0x5f,0x0,0xef,0xff,0x11,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x0,0x0,0xf0,0xff,0x34,0x0,0xf0,0xff,0x90,0x0,0xf0,0xff,0xbe,0x0,0xf0,0xff,0xba,0x0,0xf0,0xff,0xb1,0x0,0xef,0xff,0xb0,0x0,0xef,0xff,0xb1,0x0,0xef,0xff,0xb0,0x0,0xef,0xff,0xb1,0x0,0xef,0xff,0xb7,0x0,0xf0,0xff,0xbe,0x0,0xf0,0xff,0x9d,0x0,0xf0,0xff,0x46,0x0,0xf0,0xff,0x3,0x0,0xf0,0xff,0x0,0x0,0xef,0xff,0x0,0x0,0xf0,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0
};

static const uint8_t NUM8[] = {
0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3,0xff,0xff,0xff,0x37,0xff,0xff,0xff,0x6f,0xff,0xff,0xff,0xa3,0xff,0xff,0xff,0xc6,0xff,0xff,0xff,0xe6,0xff,0xff,0xff,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xd8,0xff,0xff,0xff,0xb8,0xff,0xff,0xff,0x91,0xff,0xff,0xff,0x5b,0xff,0xff,0xff,0x22,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3d,0xff,0xff,0xff,0x9e,0xff,0xff,0xff,0xec,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd4,0xff,0xff,0xff,0x82,0xff,0xff,0xff,0x20,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1,0xff,0xff,0xff,0x50,0xff,0xff,0xff,0xd2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xad,0xff,0xff,0xff,0x2a,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x28,0xff,0xff,0xff,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0x94,0xff,0xff,0xff,0xc,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x6a,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdb,0xff,0xff,0xff,0x32,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3,0xff,0xff,0xff,0x99,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xff,0xff,0xff,0x4b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3,0xff,0xff,0xff,0xa5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0x4b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x8a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0xff,0xff,0xff,0xb2,0xff,0xff,0xff,0x91,0xff,0xff,0xff,0x80,0xff,0xff,0xff,0x80,0xff,0xff,0xff,0x8f,0xff,0xff,0xff,0xaf,0xff,0xff,0xff,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xff,0xff,0xff,0x35,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x52,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xff,0xff,0xff,0x91,0xff,0xff,0xff,0x33,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1f,0xff,0xff,0xff,0x76,0xff,0xff,0xff,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdd,0xff,0xff,0xff,0xf,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x12,0xff,0xff,0xff,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa3,0xff,0xff,0xff,0x17,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4,0xff,0xff,0xff,0x6e,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x98,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x96,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0x62,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x24,0xff,0xff,0xff,0xd6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x30,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1d,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x15,0xff,0xff,0xff,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xac,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x85,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1c,0xff,0xff,0xff,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0x22,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd2,0xff,0xff,0xff,0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7d,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x34,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xba,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd1,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x74,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xff,0xff,0xff,0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x40,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x12,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x90,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xdd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4b,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xcf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x52,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x74,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x28,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x54,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x96,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x10,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x28,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xad,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb5,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x14,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2e,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xcb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x5e,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x96,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x9e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa3,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x71,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x64,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0xd,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x6d,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x3b,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x77,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xab,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0x6,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0x1a,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa8,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x5f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbf,0xff,0xff,0xff,0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x5b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x49,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x6,0xff,0xff,0xff,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x99,0xff,0xff,0xff,0x1,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xcf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xda,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x5e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x9c,0xff,0xff,0xff,0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x60,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1,0xff,0xff,0xff,0xc1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xba,0xff,0xff,0xff,0x16,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0xdf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd6,0xff,0xff,0xff,0x4,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x21,0xff,0xff,0xff,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xff,0xff,0xff,0x4d,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x8d,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x42,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x4f,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xac,0xff,0xff,0xff,0x25,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4d,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x94,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x71,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0x97,0xff,0xff,0xff,0x19,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x2a,0xff,0xff,0xff,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd6,0xff,0xff,0xff,0x9,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x78,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xff,0xff,0xff,0x9a,0xff,0xff,0xff,0x28,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1c,0xff,0xff,0xff,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0xff,0xff,0xff,0x29,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x64,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0xb4,0xff,0xff,0xff,0x46,0xff,0xff,0xff,0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x20,0xff,0xff,0xff,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xff,0xff,0xff,0x45,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x49,0xff,0xff,0xff,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xff,0xff,0xff,0x7a,0xff,0xff,0xff,0x40,0xff,0xff,0xff,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x5b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1e,0xff,0xff,0xff,0xc7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x5b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0x7c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xff,0xff,0xff,0x4d,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x35,0xff,0xff,0xff,0xc9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0x2d,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x8,0xff,0xff,0xff,0x8e,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0x89,0xff,0xff,0xff,0xb,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x23,0xff,0xff,0xff,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xff,0xff,0xff,0xbe,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xff,0xff,0xff,0x4e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3f,0xff,0xff,0xff,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcc,0xff,0xff,0xff,0x28,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1f,0xff,0xff,0xff,0x83,0xff,0xff,0xff,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x99,0xff,0xff,0xff,0x9,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x4e,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8d,0xff,0xff,0xff,0x5,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4,0xff,0xff,0xff,0x50,0xff,0xff,0xff,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd6,0xff,0xff,0xff,0x29,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x52,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x64,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x2c,0xff,0xff,0xff,0x9a,0xff,0xff,0xff,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0x41,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3e,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x50,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x14,0xff,0xff,0xff,0x82,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0x4e,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1f,0xff,0xff,0xff,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x5f,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x10,0xff,0xff,0xff,0x85,0xff,0xff,0xff,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0x41,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3,0xff,0xff,0xff,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x81,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x1d,0xff,0xff,0xff,0xa8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xff,0xff,0xff,0x26,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x76,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4e,0xff,0xff,0xff,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd1,0xff,0xff,0xff,0x7,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1b,0xff,0xff,0xff,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xff,0xff,0xff,0x1e,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x18,0xff,0xff,0xff,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x86,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x98,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7a,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x6,0xff,0xff,0xff,0xa7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0x24,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1a,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3,0xff,0xff,0xff,0xaa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x80,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7f,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0xcb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0x1a,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xff,0xff,0xff,0x1d,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x28,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x7a,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x2c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc7,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x88,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcd,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x6c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x85,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x14,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x16,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xa4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4f,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xb3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4b,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x29,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x67,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x78,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x12,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x35,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x98,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x5,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x14,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x10,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb8,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x24,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xba,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4e,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x18,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xac,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xcb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x96,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xa0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x7c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x6e,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x68,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x4a,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xcd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x3d,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcc,0xff,0xff,0xff,0x1,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x37,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0x5,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x6e,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0xbc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x60,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xff,0xff,0xff,0x39,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x66,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x41,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xff,0xff,0xff,0x2d,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x3b,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xff,0xff,0xff,0x1,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x5d,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xff,0xff,0xff,0x47,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x45,0xff,0xff,0xff,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x46,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1,0xff,0xff,0xff,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0x8d,0xff,0xff,0xff,0xb,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x5,0xff,0xff,0xff,0x7c,0xff,0xff,0xff,0xfd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xad,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x1b,0xff,0xff,0xff,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xff,0xff,0xff,0x70,0xff,0xff,0xff,0xb,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x2,0xff,0xff,0xff,0x50,0xff,0xff,0xff,0xd4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xff,0xff,0xff,0x13,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x3c,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0x9d,0xff,0xff,0xff,0x50,0xff,0xff,0xff,0x11,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4,0xff,0xff,0xff,0x38,0xff,0xff,0xff,0x84,0xff,0xff,0xff,0xe0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xff,0xff,0xff,0x34,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x4e,0xff,0xff,0xff,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdd,0xff,0xff,0xff,0xb9,0xff,0xff,0xff,0xa0,0xff,0xff,0xff,0x90,0xff,0xff,0xff,0x90,0xff,0xff,0xff,0x98,0xff,0xff,0xff,0xad,0xff,0xff,0xff,0xce,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0x4a,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x44,0xff,0xff,0xff,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0xff,0xff,0xff,0x40,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x29,0xff,0xff,0xff,0xd3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd3,0xff,0xff,0xff,0x28,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x9,0xff,0xff,0xff,0x8c,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xff,0xff,0xff,0x8d,0xff,0xff,0xff,0x9,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x2c,0xff,0xff,0xff,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb8,0xff,0xff,0xff,0x2c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x31,0xff,0xff,0xff,0xa4,0xff,0xff,0xff,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xff,0xff,0xff,0xa5,0xff,0xff,0xff,0x31,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0xf,0xff,0xff,0xff,0x62,0xff,0xff,0xff,0xae,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xff,0xff,0xff,0xae,0xff,0xff,0xff,0x65,0xff,0xff,0xff,0xf,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x4,0xff,0xff,0xff,0x32,0xff,0xff,0xff,0x64,0xff,0xff,0xff,0x92,0xff,0xff,0xff,0xb3,0xff,0xff,0xff,0xcf,0xff,0xff,0xff,0xe4,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xf9,0xff,0xff,0xff,0xf8,0xff,0xff,0xff,0xf0,0xff,0xff,0xff,0xe3,0xff,0xff,0xff,0xcf,0xff,0xff,0xff,0xb1,0xff,0xff,0xff,0x92,0xff,0xff,0xff,0x64,0xff,0xff,0xff,0x32,0xff,0xff,0xff,0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0,0xff,0xff,0xff,0x0
};


static const uint8_t pointB[] = {
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xe7,0xe7,0xe7,0xff,0xaa,0xaa,0xaa,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0xe4,0xe4,0xe4,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaa,0xaa,0xaa,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,0xec,0xec,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x90,0x90,0x90,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xf7,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9a,0x9a,0x9a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9d,0x9d,0x9d,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xf0,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa7,0xa7,0xa7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xab,0xab,0xab,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb5,0xb5,0xb5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb8,0xb8,0xb8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc2,0xc2,0xc2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc5,0xc5,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xec,0xec,0xec,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xf0,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd3,0xd3,0xd3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdd,0xdd,0xdd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe0,0xe0,0xe0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xfb,0xfb,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x94,0x94,0x94,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x98,0x98,0x98,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xf7,0xf7,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa1,0xa1,0xa1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa5,0xa5,0xa5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb2,0xb2,0xb2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xf7,0xf7,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc0,0xc0,0xc0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xfd,0xfd,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xcd,0xcd,0xcd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd1,0xd1,0xd1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdb,0xdb,0xdb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe8,0xe8,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xec,0xec,0xec,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x92,0x92,0x92,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xfd,0xfd,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x96,0x96,0x96,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0xa0,0xa0,0xa0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0xa3,0xa3,0xa3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0xad,0xad,0xad,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0xb1,0xb1,0xb1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0x8e,0x8e,0x8e,0xff,0xba,0xba,0xba,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0x8e,0x8e,0x8e,0xff,0xbe,0xbe,0xbe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0x8e,0x8e,0x8e,0xff,0xc8,0xc8,0xc8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0x8e,0x8e,0x8e,0xff,0xcb,0xcb,0xcb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0x8e,0x8e,0x8e,0xff,0xd5,0xd5,0xd5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0x8e,0x8e,0x8e,0xff,0xd9,0xd9,0xd9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0x8e,0x8e,0x8e,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfc,0xfc,0xfc,0xff,0x8e,0x8e,0x8e,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xfd,0xfd,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfd,0xfd,0xfd,0xff,0x8e,0x8e,0x8e,0xff,0xf0,0xf0,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0x8e,0x8e,0x8e,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0x8e,0x8e,0x8e,0xff,0xfe,0xfe,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};


static const uint8_t pointA[] = {
0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf6,0xf6,0xf6,0xff,0xe4,0xe4,0xe4,0xff,0xb6,0xb6,0xb6,0xff,0xf7,0xf7,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x98,0x98,0x98,0xff,0xda,0xda,0xda,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xf7,0xf7,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb4,0xb4,0xb4,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xf6,0xf6,0xf6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x97,0x97,0x97,0xff,0xd8,0xd8,0xd8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfa,0xfa,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x93,0x93,0x93,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9a,0x9a,0x9a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9f,0x9f,0x9f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0xf1,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa5,0xa5,0xa5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xab,0xab,0xab,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb1,0xb1,0xb1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb7,0xb7,0xb7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0xe8,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc9,0xc9,0xc9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xcf,0xcf,0xcf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd5,0xd5,0xd5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdb,0xdb,0xdb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x94,0x94,0x94,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9a,0x9a,0x9a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0xe8,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe8,0xe8,0xe8,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa0,0xa0,0xa0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa6,0xa6,0xa6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xac,0xac,0xac,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0xf1,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb1,0xb1,0xb1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb8,0xb8,0xb8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd6,0xd6,0xd6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdc,0xdc,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xde,0xde,0xde,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe8,0xe8,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdc,0xdc,0xdc,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9a,0x9a,0x9a,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd7,0xd7,0xd7,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa1,0xa1,0xa1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa6,0xa6,0xa6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xac,0xac,0xac,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb2,0xb2,0xb2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb8,0xb8,0xb8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbe,0xbe,0xbe,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xd0,0xd0,0xd0,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc4,0xc4,0xc4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xcb,0xcb,0xcb,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd6,0xd6,0xd6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdc,0xdc,0xdc,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe8,0xe8,0xe8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xca,0xca,0xca,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc5,0xc5,0xc5,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf1,0xf1,0xf1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xfa,0xfa,0xfa,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x8f,0x8f,0x8f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xfb,0xfb,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xc3,0xc3,0xc3,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa1,0xa1,0xa1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbf,0xbf,0xbf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf7,0xf7,0xf7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa7,0xa7,0xa7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xad,0xad,0xad,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb3,0xb3,0xb3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xb9,0xb9,0xb9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xbf,0xbf,0xbf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xbd,0xbd,0xbd,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xc5,0xc5,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb9,0xb9,0xb9,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xdd,0xdd,0xdd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb6,0xb6,0xb6,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xb4,0xb4,0xb4,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdf,0xdf,0xdf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xfb,0xfb,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x8f,0x8f,0x8f,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe2,0xe2,0xe2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe5,0xe5,0xe5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xaf,0xaf,0xaf,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xae,0xae,0xae,0xff,0x8e,0x8e,0x8e,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xa8,0xa8,0xa8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xad,0xad,0xad,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xb4,0xb4,0xb4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xb9,0xb9,0xb9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xee,0xee,0xee,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf0,0xf0,0xf0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xbf,0xbf,0xbf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa9,0xa9,0xa9,0xff,0x8e,0x8e,0x8e,0xff,0xc5,0xc5,0xc5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf4,0xf4,0xf4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa8,0xa8,0xa8,0xff,0x8e,0x8e,0x8e,0xff,0xca,0xca,0xca,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xcb,0xcb,0xcb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xd0,0xd0,0xd0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xd1,0xd1,0xd1,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xd7,0xd7,0xd7,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xdd,0xdd,0xdd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xde,0xde,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf9,0xf9,0xf9,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf3,0xf3,0xf3,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xeb,0xeb,0xeb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xa2,0xa2,0xa2,0xff,0x8e,0x8e,0x8e,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xed,0xed,0xed,0xff,0xff,0xff,0xff,0xff,0x9c,0x9c,0x9c,0xff,0x8e,0x8e,0x8e,0xff,0xf2,0xf2,0xf2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x8e,0x8e,0x8e,0xff,0xf5,0xf5,0xf5,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x8e,0x8e,0x8e,0xff,0xf8,0xf8,0xf8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe9,0xe9,0xe9,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x8e,0x8e,0x8e,0xff,0xfb,0xfb,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x8e,0x8e,0x8e,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x90,0x90,0x90,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x95,0x95,0x95,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x96,0x96,0x96,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe7,0xe7,0xe7,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x9b,0x9b,0x9b,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0x9c,0x9c,0x9c,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe6,0xe6,0xe6,0xff,0xff,0xff,0xff,0xff,0x9b,0x9b,0x9b,0xff,0xa2,0xa2,0xa2,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0x96,0x96,0x96,0xff,0xa8,0xa8,0xa8,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xa9,0xa9,0xa9,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xae,0xae,0xae,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe4,0xe4,0xe4,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xaf,0xaf,0xaf,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xdf,0xdf,0xdf,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xb4,0xb4,0xb4,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xb6,0xb6,0xb6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xba,0xba,0xba,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe1,0xe1,0xe1,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xbd,0xbd,0xbd,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe0,0xe0,0xe0,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xc0,0xc0,0xc0,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xff,0xff,0xff,0xff,0x95,0x95,0x95,0xff,0xc3,0xc3,0xc3,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xef,0xef,0xef,0xff,0xfa,0xfa,0xfa,0xff,0x99,0x99,0x99,0xff,0xea,0xea,0xea,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xe3,0xe3,0xe3,0xff,0xd6,0xd6,0xd6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};


r_wm_Window_t loc_WmWindow;
r_wm_Window_t loc_WmWindow1;
r_wm_Window_t loc_WmWindow2;
static r_wm_Window_t loc_SpriteWindow;


/***********************************************************

  Variable: loc_Drw2dDev

  Pointer to the Drw2D device handle
*/

static r_drw2d_Device_t loc_Drw2dDev;
static r_drw2d_Device_t loc_Drw2dDev1;

/***********************************************************
  Variable: loc_Fb

  Framebuffer information for Drw2D
*/
static r_drw2d_Framebuffer_t loc_Fb;
static r_drw2d_Framebuffer_t loc_Fb1;
/***********************************************************
  Variable: g_BlackKingBmp

  the Bitmap (external)
*/
extern const uint8_t g_BlackKingBmp[LOC_KING_SIZE];
extern const uint8_t g_GearBmp[LOC_GEAR_SIZE];

/***********************************************************
  Variable: loc_lRAM_heap, loc_VRAM_heap

  control structures for CDI heaps
*/
extern r_cdi_Heap_t       loc_lRAM_heap;
extern r_cdi_Heap_t       loc_VRAM_heap;

/***********************************************************
  Variable: loc_DHDDev

  DHD Drawing handle
*/
static struct r_drw2d_DeviceDHD_s loc_DHDDev;

/***********************************************************
  Variable: loc_ExitFlag

  flag to test exit pathes
*/
static int8_t loc_ExitFlag = R_FALSE;

/***********************************************************
  Variable: loc_GPU_Memory_Addr

  dedicated memory area for GPU (DaveHD)
*/
extern uint32_t loc_GPU_Memory_Addr;


/***********************************************************
  Section: Local Functions
*/

/***********************************************************
 Function: loc_DRW2DErrorHandler

 Error handler used for Drw2d errors
*/
static r_drw2d_Boolean_t loc_DRW2DErrorHandler(r_drw2d_Error_t  Error, void *UserData)
{
    loc_Error(1);
    return R_TRUE;
}


/***********************************************************
  Section: Local Functions
*/

/*******************************************************************************
  Function: locBtnOkPress + locRotaryCenterPress

  Button and Rotary press callback functions
*/
static void locBtnOkPress(void) {
    loc_ExitFlag = R_TRUE;
}

static void locRotaryCenterPress(void) {
    loc_ExitFlag = R_TRUE;
}


/*******************************************************************************
  Function: loc_RenderClear

  Render function called each frame
*/
static void loc_RenderClear(int32_t Frame)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;

    /* Set green-ish clear color and fill framebuffer */
    ret |= R_DRW2D_CtxBgColor(loc_Drw2dDev, 0x00000000);
    ret |= R_DRW2D_FramebufferClear(loc_Drw2dDev);

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}

 /* Function: loc_RenderClear*/

 /* Render function called each frame
*/
static void loc_RenderClear1(int32_t Frame)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;

    /* Set green-ish clear color and fill framebuffer */
    ret |= R_DRW2D_CtxBgColor(loc_Drw2dDev1, 0x00000000);
    ret |= R_DRW2D_FramebufferClear(loc_Drw2dDev1);

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}
/*******************************************************************************
  Function: loc_RenderPrimitives

  Render function called each frame
*/
static void loc_RenderPrimitives(int32_t Frame)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Point_t triangle[3];

    /* triangle */
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev, R_DRW2D_BLENDMODE_SRC);
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev, R_DRW2D_FILLMODE_SOLID);
    ret |= R_DRW2D_CtxFgColor(loc_Drw2dDev, 0xffff0000u);
    triangle[0].X = R_DRW2D_2X(4);
    triangle[0].Y = R_DRW2D_2X(10);
    triangle[1].X = R_DRW2D_2X(66);
    triangle[1].Y = R_DRW2D_2X(124);
    triangle[2].X = R_DRW2D_2X(100);
    triangle[2].Y = R_DRW2D_2X(10);
    ret |= R_DRW2D_DrawTriangles(loc_Drw2dDev, triangle, 1, 0);

    /* rect */
    ret |= R_DRW2D_CtxFgColor(loc_Drw2dDev, 0xff0000ffu);
    rect.Pos.X       = R_DRW2D_2X((  - 100) / 2);
    rect.Pos.Y       = R_DRW2D_2X(200);
    rect.Size.Width  = R_DRW2D_2X(100);
    rect.Size.Height = R_DRW2D_2X(100);
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev, &rect);

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}

/*******************************************************************************
  Function: loc_RenderTexture_intROM

  Render function called each frame
*/

#if 0
static void loc_RenderTexture_intROM(int32_t Frame)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Texture_t tex_argb32;

    tex_argb32.Handle             = R_NULL;
    tex_argb32.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;
    tex_argb32.Buffer.Pitch       = LOC_GEAR_W;
    tex_argb32.Buffer.Size.Width  = LOC_GEAR_W;
    tex_argb32.Buffer.Size.Height = LOC_GEAR_H;
    tex_argb32.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;
    tex_argb32.Buffer.Data        = (void*)g_GearBmp;

    rect.Pos.X       = R_DRW2D_2X(130);
    rect.Pos.Y       = R_DRW2D_2X(24);
    rect.Size.Width  = R_DRW2D_2X(LOC_GEAR_W);
    rect.Size.Height = R_DRW2D_2X(LOC_GEAR_H);
	
	static unsigned int rotate = 0;
	if(rotate >= 360) rotate=0;
	else rotate++;

    /* Set texture */
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev, 0, &tex_argb32);
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev, R_DRW2D_FILLMODE_TEXTURE);
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev, R_DRW2D_BLENDMODE_SRC_OVER);

    /* rotate and translate */
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/2);
    ret |= R_DRW2D_CtxTextureRotate(loc_Drw2dDev, R_DRW2D_2X(rotate));
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev, -(rect.Size.Width/2), -(rect.Size.Height/2));

    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/2, 0);
    ret |= R_DRW2D_CtxRotate(loc_Drw2dDev, R_DRW2D_2X(rotate));
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev, -(rect.Pos.X+rect.Size.Width/2), -(rect.Pos.Y+rect.Size.Height/2), 0);


    /* Draw texture into rectangle */
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev, &rect);
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev);

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}

#else


static const uint8_t locImgData[] ={0};



static void loc_RenderTexture_intROM(int32_t Frame)
{
    static unsigned int rotate = 0;
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Texture_t tex_argb32;
     r_drw2d_EffectStage_t Effects[1];

    tex_argb32.Handle             = R_NULL;
    tex_argb32.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;
    tex_argb32.Buffer.Pitch       =800 ;//LOC_GEAR_W;
    tex_argb32.Buffer.Size.Width  =800;//LOC_GEAR_W;
    tex_argb32.Buffer.Size.Height = 480;//LOC_GEAR_H;
    tex_argb32.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;//R_DRW2D_PIXELFORMAT_ARGB8888;
    tex_argb32.Buffer.Data        = (void*)locImgData;//(void*)g_GearBmp;   30912512+0x40000000

    rect.Pos.X       = R_DRW2D_2X(0);
    rect.Pos.Y       = R_DRW2D_2X(0);
    rect.Size.Width  = R_DRW2D_2X(800/*LOC_GEAR_W*/);
    rect.Size.Height = R_DRW2D_2X(480/*LOC_GEAR_H*/);
	
	
	if(rotate >= 360) rotate=0;
	else
	rotate++;
	

    /* Set texture */
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev, R_DRW2D_FILLMODE_TEXTURE/*R_DRW2D_FILLMODE_TEXTURE*/);
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev, R_DRW2D_BLENDMODE_SRC_OVER/*R_DRW2D_BLENDMODE_SRC_OVER*/);
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev, 0, &tex_argb32);

    /* rotate and translate */
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/2);
    ret |= R_DRW2D_CtxTextureRotate(loc_Drw2dDev, R_DRW2D_2X(rotate));
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev, -(rect.Size.Width/2), -(rect.Size.Height/2));

    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/2, 0);
    ret |= R_DRW2D_CtxRotate(loc_Drw2dDev, R_DRW2D_2X(rotate));
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev, -(rect.Pos.X+rect.Size.Width/2), -(rect.Pos.Y+rect.Size.Height/2), 0);


    /*
    Effects[0].Name = R_DRW2D_EFFECT_REPLACE; 
    Effects[0].Args[0].Source = R_DRW2D_EFFECT_SOURCE_TEXTURE_UNIT;
    Effects[0].Args[0].Param.Color.Source.TextureUnit = 0;
    Effects[0].Args[0].Param.Color.Operand = R_DRW2D_EFFECT_COLOR_OPERAND_RGBA; 
    Effects[1].Name = R_DRW2D_EFFECT_CONSTANT_ALPHA; 
    Effects[1].Args[0].Source = R_DRW2D_EFFECT_SOURCE_CONSTANT_COLOR;
    Effects[1].Args[0].Param.Constant = R_DRW2D_2X(0.5f); 
    ret |= R_DRW2D_CtxEffectsSet(loc_Drw2dDev, &Effects[0], 2);
    */



    /* Draw texture into rectangle */
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev, &rect);
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev);

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}
#endif

/*******************************************************************************
  Function: loc_RenderTexture_SFlash
  
  Render function called each frame
*//* bitmap widths */

#define LOC_KING_W    (30)
#define LOC_GEAR_W    (104)
#define LOC_POINTER_W (16)  // (190)
#define LOC_POINTER_W1 (10)  // (190)
#define LOC_POINTER_W2 (54)  // (190)
/* bitmap heights */
#define LOC_KING_H    (30)
#define LOC_GEAR_H    (104)
#define LOC_POINTER_H   (208)// (44)
#define LOC_POINTER_H1   (80)// (44)
#define LOC_POINTER_H2   (86)// (44)


float   dfdfdfdfdfd;
float   dfdfdfdfdfd1;

/***************************���������Ծ���***************************************
r_drw2d_UVCoord_t uv = {
                         { R_DRW2D_2U(0), R_DRW2D_2V(0) },
			 { R_DRW2D_2U(1), R_DRW2D_2V(0) },
			 { R_DRW2D_2U(1), R_DRW2D_2V(1) },
			 { R_DRW2D_2U(0), R_DRW2D_2V(1) }
                       };
************************************************************************/

/*******************************************************************************
  Function: loc_RenderTexture_intROM

  Render function called each frame
*/
static void loc_RenderTexture_intROM111(int32_t Frame)//���뵭��Ч��
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Texture_t tex_argb32;
    uint32_t xPos, yPos;
    r_drw2d_EffectStage_t effect[2];
    r_drw2d_UVCoord_t uv[] = { { R_DRW2D_2U(0), R_DRW2D_2V(0), },
			     { R_DRW2D_2U(1), R_DRW2D_2V(0) ,},
			     { R_DRW2D_2U(1), R_DRW2D_2V(1), },
			     { R_DRW2D_2U(0), R_DRW2D_2V(1), } };
   

    
    tex_argb32.Handle             = R_NULL;
    tex_argb32.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;
    tex_argb32.Buffer.Pitch       = LOC_GEAR_W;
    tex_argb32.Buffer.Size.Width  = LOC_GEAR_W;
    tex_argb32.Buffer.Size.Height = LOC_GEAR_H;
    tex_argb32.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;
    tex_argb32.Buffer.Data        = (void*)g_GearBmp;

    rect.Pos.X       = R_DRW2D_2X(0);
    rect.Pos.Y       = R_DRW2D_2X(0);
    rect.Size.Width  = R_DRW2D_2X(LOC_GEAR_W);
    rect.Size.Height = R_DRW2D_2X(LOC_GEAR_H);

    xPos = R_DRW2D_2X(0);
    yPos = R_DRW2D_2X(0);
    /* Set texture */
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);    
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev1, 0, &tex_argb32);
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1, xPos, yPos);
 

    //////////////
effect[0].Name = R_DRW2D_EFFECT_REPLACE;
effect[0].Args[0].Source = R_DRW2D_EFFECT_SOURCE_TEXTURE_UNIT;
effect[0].Args[0].Param.Color.Source.TextureUnit = 0;/* Same value as R_DRW2D_CtxTextureSet */
effect[0].Args[0].Param.Color.Operand = R_DRW2D_EFFECT_COLOR_OPERAND_RGBA;
effect[1].Name = R_DRW2D_EFFECT_CONSTANT_ALPHA;
effect[1].Args[0].Source = R_DRW2D_EFFECT_SOURCE_CONSTANT;
effect[1].Args[0].Param.Constant = R_DRW2D_2X(0.01f*Frame); /* const alpha value 0.0f-1.0f */
ret |= R_DRW2D_CtxEffectsSet(loc_Drw2dDev1, effect, 2);


ret |= R_DRW2D_DrawRectUV(loc_Drw2dDev1, &rect, &uv);
ret |= R_DRW2D_CtxEffectsDelete(loc_Drw2dDev1); /* Disable effect */
 

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}



static void loc_RenderTexture_Pointer1(int32_t Frame)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Rect_t  rect1;
    r_drw2d_Rect_t  rect2;    
    r_drw2d_Texture_t tex_argb32,tex_argb33,tex_argb34;



//r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
//r_drw2d_Rect_t  rect;
//uint32_t             PosX = 100;
//uint32_t             PosY = 50;
//uint32_t             TextureWidth = 50;
//uint32_t             TextureHeight= 50;
//r_drw2d_EffectStage_t   effect[2];



        
 //	���õ����ڴ����ڵ�����
    ///////////////////////////////// 0 1
    tex_argb32.Handle             = R_NULL;
    tex_argb32.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;//
    tex_argb32.Buffer.Pitch       = LOC_POINTER_W;
    tex_argb32.Buffer.Size.Width  = LOC_POINTER_W;
    tex_argb32.Buffer.Size.Height = LOC_POINTER_H;
    tex_argb32.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;  //R_DRW2D_PIXELFORMAT_RGB565;// 
    tex_argb32.Buffer.Data        = (void*)pointA;//0x1060ad00;  //g_GearBmp;
    
    rect.Pos.X         = R_DRW2D_2X(300);//���õ��Ǿ������Ͻǵ�����
    rect.Pos.Y         = R_DRW2D_2X(200);
    rect.Size.Width  = R_DRW2D_2X(LOC_POINTER_W);  //192
    rect.Size.Height = R_DRW2D_2X(LOC_POINTER_H);  //48


 //	���õ����ڴ����ڵ�����
    ///////////////////////////////// 0 1
    tex_argb33.Handle             = R_NULL;
    tex_argb33.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;//
    tex_argb33.Buffer.Pitch       = LOC_POINTER_W1;
    tex_argb33.Buffer.Size.Width  = LOC_POINTER_W1;
    tex_argb33.Buffer.Size.Height = LOC_POINTER_H1;
    tex_argb33.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;  //R_DRW2D_PIXELFORMAT_RGB565;// 
    tex_argb33.Buffer.Data        = (void*)pointB;//0x1060e100;  //g_GearBmp;
    
    rect1.Pos.X         = R_DRW2D_2X(300);//���õ��Ǿ������Ͻǵ�����
    rect1.Pos.Y         = R_DRW2D_2X(330);
    rect1.Size.Width  = R_DRW2D_2X(LOC_POINTER_W1);  //192
    rect1.Size.Height = R_DRW2D_2X(LOC_POINTER_H1);  //48




 //	���õ����ڴ����ڵ�����
    ///////////////////////////////// 
    tex_argb34.Handle             = R_NULL;
    tex_argb34.Flags              = R_DRW2D_TEX_VT | R_DRW2D_TEX_BILINEAR;//
    tex_argb34.Buffer.Pitch       = LOC_POINTER_W2;
    tex_argb34.Buffer.Size.Width  = LOC_POINTER_W2;
    tex_argb34.Buffer.Size.Height = LOC_POINTER_H2;
    tex_argb34.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;  //R_DRW2D_PIXELFORMAT_RGB565;// 
    tex_argb34.Buffer.Data        = (void*)NUM8;//0x10601b00;  //g_GearBmp;
    
    rect2.Pos.X         = R_DRW2D_2X(0);//���õ��Ǿ������Ͻǵ�����
    rect2.Pos.Y         = R_DRW2D_2X(0);
    rect2.Size.Width  = R_DRW2D_2X(LOC_POINTER_W2);  //192
    rect2.Size.Height = R_DRW2D_2X(LOC_POINTER_H2);  //48



      
    
    /* Set texture *////rect
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev1, 0, &tex_argb32);//����Դ����0
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev1, R_DRW2D_FILLMODE_TEXTURE);//Ϊԭ��������ģʽ
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev1, R_DRW2D_BLENDMODE_SRC_OVER  );//���� Ԥ�����ɫ/������ Դ/Ŀ���Ϸ���

////////////////////////////////////////////////////
	/* rotate and translate */
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);//����texture����
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/9);//�Ե�ǰ��ԭ�������ת�������
    ret |= R_DRW2D_CtxTextureRotate(loc_Drw2dDev1, -R_DRW2D_2X(Frame));
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1,-(1*(rect.Size.Width/2)), -(rect.Size.Height/9)); //�Ե�ǰ��ԭ�������ת�������

    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);////���ö���任����
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev1, rect.Pos.X+rect.Size.Width/2, rect.Pos.Y+rect.Size.Height/9, 0);//�Ե�ǰ�Ķ��������ת�������
    ret |= R_DRW2D_CtxRotate(loc_Drw2dDev1, -R_DRW2D_2X(Frame));//�Ե�ǰ�Ķ����������ת����� +����˳ʱ�룬-������ʱ��+R_DRW2D_2X(Frame)
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev1, -(rect.Pos.X+(1*rect.Size.Width/2)),-(rect.Pos.Y+rect.Size.Height/9), 0);//�Ե�ǰ�Ķ��������ת�������
 
    /* Draw texture into rectangle */
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev1, &rect);//����һ������
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);//����texture����
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);//���ö���任����
///////////////////////////////////////////////////////////////////////////////



//��ӵ�



//////////////////////////////////////////////////////////////////////////////////////////
    /* Set texture *////rect1
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev1, 0, &tex_argb33);//����Դ���� 000
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev1, R_DRW2D_FILLMODE_TEXTURE);//Ϊԭ��������ģʽ
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev1, R_DRW2D_BLENDMODE_SRC_OVER );//���� Ԥ�����ɫ/������ Դ/Ŀ���Ϸ���

    ////////////////////////////////////////////////////
	/* rotate and translate */
   ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);//����texture����
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1, rect1.Pos.X+rect1.Size.Width/2, rect1.Pos.Y+rect1.Size.Height/8);//�Ե�ǰ��ԭ�������ת�������
     ret |= R_DRW2D_CtxTextureRotate(loc_Drw2dDev1, R_DRW2D_2X(Frame));
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1,-(1*(rect1.Size.Width/2)), -(rect1.Size.Height/8)); //�Ե�ǰ��ԭ�������ת�������

     ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);////���ö���任����
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev1, rect1.Pos.X+rect1.Size.Width/2, rect1.Pos.Y+rect1.Size.Height/8, 0);//�Ե�ǰ�Ķ��������ת�������
   ret |= R_DRW2D_CtxRotate(loc_Drw2dDev1, R_DRW2D_2X(Frame));//�Ե�ǰ�Ķ����������ת����� +����˳ʱ�룬-������ʱ��+R_DRW2D_2X(Frame)
    ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev1, -(rect1.Pos.X+(1*rect1.Size.Width/2)),-(rect1.Pos.Y+rect1.Size.Height/8), 0);//�Ե�ǰ�Ķ��������ת�������
 	
    /* Draw texture into rectangle */
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev1, &rect1);//����һ������
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);//����texture����
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);//���ö���任����
    ////////////////////////////////////////






     

    /* Set texture *////rect
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev1, 0, &tex_argb34);//����Դ����0
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev1, R_DRW2D_FILLMODE_TEXTURE);//Ϊԭ��������ģʽ
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev1, R_DRW2D_BLENDMODE_SRC_OVER );//���� Ԥ�����ɫ/������ Դ/Ŀ���Ϸ���

/////////////////////����/////////////////////////////////////////////////////////////////////////////////////////////////////////////
  ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);
   ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev1, R_DRW2D_2X(100), R_DRW2D_2X(100));
   ret |= R_DRW2D_CtxTextureScale( loc_Drw2dDev1, R_DRW2D_2X(dfdfdfdfdfd1), R_DRW2D_2X(dfdfdfdfdfd1));

   ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);
   ret |= R_DRW2D_CtxTranslate(loc_Drw2dDev1, R_DRW2D_2X(100), R_DRW2D_2X(100), 0);
   ret |= R_DRW2D_CtxScale( loc_Drw2dDev1,R_DRW2D_2X(dfdfdfdfdfd),  R_DRW2D_2X(dfdfdfdfdfd), R_DRW2D_2X(1));
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 

   /* Draw texture into rectangle */
    ret |= R_DRW2D_DrawRect(loc_Drw2dDev1, &rect2);//����һ������
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev1);//����texture����
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev1);//���ö���任����
///////////////////////////////////////////////////////////////////////////////






     
     /*********************************************/
    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}

static void loc_RenderTexture_SFlash_Back(int32_t Frame, void* TextureAddr)
{
    r_drw2d_Error_t ret = R_DRW2D_ERR_OK;
    r_drw2d_Rect_t  rect;
    r_drw2d_Texture_t tex_argb32;
    uint32_t xPos, yPos;

    tex_argb32.Handle             = R_NULL;
    tex_argb32.Flags              = (r_drw2d_TextureFlags_t)0;
    tex_argb32.Buffer.Pitch       = 800;
    tex_argb32.Buffer.Size.Width  = 800;
    tex_argb32.Buffer.Size.Height = 480;
    tex_argb32.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_RGB565;// R_DRW2D_PIXELFORMAT_ARGB8888;
    tex_argb32.Buffer.Data        = (void*)0x10233a00;//TextureAddr;

    /* Set texture */
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxIdentity(loc_Drw2dDev);
    ret |= R_DRW2D_CtxTextureSet(loc_Drw2dDev, 0, &tex_argb32);
    ret |= R_DRW2D_CtxFillMode(loc_Drw2dDev, R_DRW2D_FILLMODE_TEXTURE);
    ret |= R_DRW2D_CtxBlendMode(loc_Drw2dDev, R_DRW2D_BLENDMODE_SRC_OVER);
    xPos = R_DRW2D_2X(0);
    yPos = R_DRW2D_2X(0);
    ret |= R_DRW2D_CtxTextureTranslate(loc_Drw2dDev, xPos, yPos);

    /* Draw texture into rectangle */
    rect.Pos.X       = R_DRW2D_2X(0);
    rect.Pos.Y       = R_DRW2D_2X(0);
    rect.Size.Width  = R_DRW2D_2X(800);
    rect.Size.Height = R_DRW2D_2X(480);

    ret |= R_DRW2D_DrawRect(loc_Drw2dDev, &rect);
    ret |= R_DRW2D_CtxTextureIdentity(loc_Drw2dDev);

    if (R_DRW2D_ERR_OK != ret)
    {
       loc_Error(ret);
    }
}

/*******************************************************************************
  Function: loc_App

  The render loop
*/
 uint16_t  fdfdaf;
 #define LOC_SF_ADDRESS_EX (0x100bb800)
static void loc_App(void)
{
    uint32_t frame;
    r_wm_Sprite_t sprite;
    int32_t ret = R_DRW2D_ERR_OK;
    r_wm_Error_t wmErr;

    void*   textureAddr = R_NULL;
#if (LOC_VOWE_ENABLE == 1) && (LOC_DL_CHANGE != 0)
    int32_t dl_cnt = 1;
#endif /* LOC_DL_CHANGE */
  /*#if (LOC_EX_MEM_TYPE == 1)
    textureAddr = loc_StartOCTA();
  #elif (LOC_EX_MEM_TYPE == 2)

  
    textureAddr = loc_StartHYPB();
  #else*/
    //textureAddr = loc_StartSFMA();//////////////////////////////////////////////
    //textureAddr = InitSeriFlashExternReadMode();
  //#endif
    /* Init framebuffer struct */
    loc_Fb.Flags  = (r_drw2d_FramebufferFlags_t)0;
    loc_Fb.Handle = 0; /* not used currently */
    loc_Fb.Buffer.Pitch       = LOC_DISPLAY_STRIDE;
    loc_Fb.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_RGB565;//R_DRW2D_PIXELFORMAT_ARGB8888 ;
    loc_Fb.Buffer.Size.Width=  800 ;
    loc_Fb.Buffer.Size.Height = LOC_DISPLAY_HEIGHT;
    loc_Fb.Buffer.Data        = 0;

    loc_Fb1.Flags  = (r_drw2d_FramebufferFlags_t)0;
    loc_Fb1.Handle = 0; /* not used currently */
    loc_Fb1.Buffer.Pitch       = 640;
    loc_Fb1.Buffer.PixelFormat = R_DRW2D_PIXELFORMAT_ARGB8888;
    loc_Fb1.Buffer.Size.Width  = 600;
    loc_Fb1.Buffer.Size.Height = 480;
    loc_Fb1.Buffer.Data        = 0;
    
    //loc_InitWM(&loc_lRAM_heap, &loc_VRAM_heap);
    //wmErr = R_WM_FrameEndMark(LOC_WM_UNIT, 0);
    //if(R_WM_ERR_OK != wmErr)
    //{
    //   loc_Error(1);
    //}
    //wmErr = R_WM_FrameWait(LOC_WM_UNIT, 0);
    //if(R_WM_ERR_OK != wmErr)
    //{
    //   loc_Error(1);
    //}

    R_BSP_STDIO_Printf("Now using VOWE, JCUA and WM\n");

    #if LOC_VOWE_ENABLE == 1
    //loc_VoweStart();
    #endif

    Gfx_Create_Window(R_WM_WINMODE_FB,//��ͼ
                     0,
                     0,
                     0,
                     LOC_DISPLAY_WIDTH,
                     LOC_DISPLAY_HEIGHT,
                     LOC_DISPLAY_STRIDE,
		     1,
                      R_WM_COLORFMT_RGB565,//R_WM_COLORFMT_ARGB8888,                                 
                     &loc_WmWindow);

   Gfx_Create_Window(R_WM_WINMODE_FB,//���뼰�м�����
                     100,
                     0,
                     1,
                     600,//400
                     480,
                     640,//512    pitch =  128�ı����Ҵ��ڿ�����ӽ���ֵ
		     2, 
                     R_WM_COLORFMT_ARGB8888,//R_WM_COLORFMT_RGB565,//
                     &loc_WmWindow1);




   Gfx_Create_Window(R_WM_WINMODE_FB,//jpeg����
                     560,
                     0,
                     3,
                     240,
                     320,
                     256,
		     2,
                      R_WM_COLORFMT_ARGB8888,//R_WM_COLORFMT_ARGB8888,
                     &loc_WmWindow2);

   
					   
   Gfx_Create_Window(R_WM_WINMODE_SPRITES,//ABSָʾ��
                     40,
                     40,
                     2,
                     60,//LOC_KING_W,
                     20,//LOC_KING_H,
                     2048,
		     1,
                     R_WM_COLORFMT_ARGB8888,
                     &loc_SpriteWindow);

    loc_JcuaRun();

    /* assign texture and show sprite */
    sprite.Data   = (void*)ABS;
    sprite.PosX   = 0;
    sprite.PosY   = 0;
    sprite.PosZ   = 0;
    sprite.Width  = 60;//LOC_KING_W;
    sprite.Height = 20;//LOC_KING_H;
    sprite.Window = &loc_SpriteWindow;
    wmErr = R_WM_SpriteCreate(LOC_WM_UNIT, &sprite);
    if(R_WM_ERR_OK != wmErr)
    {
       loc_Error(1);
    }
    wmErr = R_WM_SpriteEnable(LOC_WM_UNIT, &sprite);
    if(R_WM_ERR_OK != wmErr)
    {
       loc_Error(1);
    }
    /* start up gfx engine */
 ret |= R_DRW2D_Init();
    ret |= R_DRW2D_GlobalErrCallbackSet(&loc_DRW2DErrorHandler, R_NULL);


    ret |= R_DRW2D_Open(LOC_DRW2D_UNIT, R_DRW2D_UNIT_DHD0, &loc_DHDDev, &loc_Drw2dDev);
    

#if (LOC_EX_MEM_TYPE == 1)
    R_BSP_STDIO_Printf("Now using WM, DRW2D, SPEA, OCTA and VOWE\n");
#elif (LOC_EX_MEM_TYPE == 2)
    R_BSP_STDIO_Printf("Now using WM, DRW2D, SPEA, HYPB and VOWE\n");
#else
    R_BSP_STDIO_Printf("Now using WM, DRW2D, SPEA, SFMA and VOWE\n");
#endif
    loc_ExitFlag = R_FALSE;
    loc_Fb.Buffer.Data = R_WM_WindowNewDrawBufGet(LOC_WM_UNIT, &loc_WmWindow);
        ret |= R_DRW2D_FramebufferSet(loc_Drw2dDev, &loc_Fb);

        /* render onto this backbuffer and wait for GPU to finish*/
        //loc_RenderClear(frame);
	//loc_RenderClear1(frame);
      //  loc_RenderPrimitives(frame);
      //  loc_RenderTexture_intROM(frame);
        loc_RenderTexture_SFlash_Back(1, LOC_SF_ADDRESS_EX);

        R_DRW2D_GpuFinish(loc_Drw2dDev, R_DRW2D_FINISH_WAIT);

        /* let WM care about the handling of the buffer swap */
       // wmErr = R_WM_WindowSwap(LOC_WM_UNIT, &loc_WmWindow);
	//wmErr = R_WM_FrameEndMark(LOC_WM_UNIT, 0);
	// wmErr = R_WM_WindowSwap(LOC_WM_UNIT, &loc_WmWindow);
	//wmErr = R_WM_FrameEndMark(LOC_WM_UNIT, 0);
        //if(R_WM_ERR_OK != wmErr)
       // {
       //    loc_Error(1);
      //  }
      //  wmErr = R_WM_FrameWait(LOC_WM_UNIT, 0);
	//while(1){;}
	
	ret |= R_DRW2D_Open(LOC_DRW2D_UNIT, R_DRW2D_UNIT_DHD0, &loc_DHDDev, &loc_Drw2dDev1);
    
    while(1) 
    {
	frame++;
	fdfdaf++;
	if(fdfdaf>50)
	{
	    fdfdaf=0;
	}
      if(fdfdaf>25)
      {
	dfdfdfdfdfd = 0.5;
	dfdfdfdfdfd1 = 2;



    wmErr = R_WM_SpriteEnable(LOC_WM_UNIT, &sprite);
    if(R_WM_ERR_OK != wmErr)
    {
	loc_Error(1);
    }	
      }      
  	 else
	 {

    wmErr = R_WM_SpriteDisable(LOC_WM_UNIT, &sprite);
    if(R_WM_ERR_OK != wmErr)
    {
       loc_Error(1);
    }

	     
	  dfdfdfdfdfd = 1;
	  dfdfdfdfdfd1 = 1; 
 }
	  
	loc_Fb1.Buffer.Data = R_WM_WindowNewDrawBufGet(LOC_WM_UNIT, &loc_WmWindow1);
        ret |= R_DRW2D_FramebufferSet(loc_Drw2dDev1, &loc_Fb1);

        loc_RenderClear1(frame);
  
	loc_RenderTexture_Pointer1(frame);
	loc_RenderTexture_intROM111(frame);
	
        R_DRW2D_GpuFinish(loc_Drw2dDev1, R_DRW2D_FINISH_WAIT);

 
        wmErr = R_WM_WindowSwap(LOC_WM_UNIT, &loc_WmWindow1);
        if(R_WM_ERR_OK != wmErr)
        {
           loc_Error(1);
        }

	if( frame>360){ frame = 0;}

        
        wmErr = R_WM_FrameEndMark(LOC_WM_UNIT, 0);
        wmErr = R_WM_FrameWait(LOC_WM_UNIT, 0);
        if(R_WM_ERR_OK != wmErr)
        {
           loc_Error(1);
        }
        if (R_DRW2D_ERR_OK != ret)
        {
            loc_Error(ret);
        }

    }
    ret |= R_WM_WindowDisable(LOC_WM_UNIT, &loc_SpriteWindow);
    ret |= R_WM_WindowDisable(LOC_WM_UNIT, &loc_WmWindow);
    ret |= R_WM_FrameEndMark(LOC_WM_UNIT, 0);
    ret |= R_WM_FrameWait(LOC_WM_UNIT, 0);
    
    R_BSP_STDIO_Printf("Closing  DRW2D and VOWE\n");
    ret |= R_DRW2D_Close(loc_Drw2dDev);
    ret |= R_DRW2D_Exit();
  #if LOC_VOWE_ENABLE == 1
    loc_VoweStop();
  #endif

    R_BSP_STDIO_Printf("Closing  WM and SPEA\n");
    ret |= R_WM_SpriteDisable(LOC_WM_UNIT, &sprite);
    ret |= R_WM_SpriteDelete(LOC_WM_UNIT, &sprite);
    ret |= R_WM_WindowDelete(LOC_WM_UNIT, &loc_SpriteWindow);
    ret |= R_WM_WindowDelete(LOC_WM_UNIT, &loc_WmWindow);
    ret |= R_WM_FrameEndMark(LOC_WM_UNIT, 0);
    ret |= R_WM_FrameWait(LOC_WM_UNIT, 0);
    ret |= R_WM_DevDeinit(LOC_WM_UNIT);

#if (LOC_EX_MEM_TYPE == 1)
    loc_StopOCTA();
#elif (LOC_EX_MEM_TYPE == 2)
    loc_StopHYPB();
#else
    //loc_StopSFMA();
#endif

    if (R_DRW2D_ERR_OK != ret)
    {
        loc_Error(ret);
    }
}

/*******************************************************************************
  Function: loc_Error

  This function is called when an error occurs.
  The application will hang.
*/

void loc_Error(int32_t Err)
{
    while (Err)
    {
    }
}


/*******************************************************************************
  Function: main

  entry after startup
*/
volatile unsigned int* SYSCKSC_ICPUCLKD_CTL = (volatile unsigned int*)(0xFFF8A100);
int32_t main(void)
{
	r_dev_PinConfig_t port_config;
	
   

    Sys_Init();







	//  开启3.3V电源	
    port_config.Port = 45;                    //  port number
    port_config.Pin = 1;                      //  pin number
    port_config.Func = 0;                     //  pin function 0: GPIO, 1-4 alternative function
    port_config.Dir = R_DEV_PIN_OUT;          //  pin direction, see <r_dev_PinDirection_t>
    port_config.Feedback = 1;                 //  1: physical (output) pin level provided to input register, 0: disabled
    port_config.Pull = R_DEV_PIN_PULLNO;      //  internal pull up configuration, see <r_dev_PinPull_t>
    port_config.OpenDrain = 0;                //  1: open drain emulation, 0: push-pull
    port_config.CurrLimit = 1;                //  1: high level current limitation, 0: low level current limitation 
    port_config.HiSpeed = 1;                  //  1: Speed limitation disabled (drive strength control off), 0: Speed limit enabled  
    port_config.InputType = R_DEV_PIN_CMOS1;  //  possible types of input characteristics, see <r_dev_PinInput_t>
	R_DEV_PinConfig(&port_config);
	R_GPIO_WritePin(45, 1, 1);
	
	//  开启61194的PWM
    port_config.Port = 3;                     //  port number
    port_config.Pin = 2;                      //  pin number
	R_DEV_PinConfig(&port_config);
	R_GPIO_WritePin(3, 2, 1);
	
	//  开启61194的EN
    port_config.Port = 3;                     //  port number
    port_config.Pin = 3;                      //  pin number
	R_DEV_PinConfig(&port_config);
	R_GPIO_WritePin(3, 3, 1);
	
	//  开启1531的EN
    port_config.Port = 3;                     //  port number
    port_config.Pin = 6;                      //  pin number
	R_DEV_PinConfig(&port_config);
	R_GPIO_WritePin(3, 6, 1);
	
	//  降低频率
	//SYSCKSC_ICPUCLKD_CTL[0] = 2u;
	
    
    //R_BSP_SetButtonCallback(BSP_BTN_OK_PRESS,       locBtnOkPress);
    //R_BSP_SetButtonCallback(BSP_BTN_CENTER_PRESS,   locRotaryCenterPress);
    
    Gfx_Sys_Start();
    //loc_SetupMemManager();//�ڴ��ʼ�� 
    //R_UTIL_DHD_Init(LOC_DHD_UNIT);
    //R_UTIL_DHD_Config((dhd_gpu_ptr_t)loc_GPU_Memory_Addr, DHD_MEMORY_SIZE, &loc_lRAM_heap);

    loc_App();
    R_UTIL_DHD_DeInit(LOC_DHD_UNIT);

    return(0);
}


